# -*- coding: utf-8 -*-
#
# Author: Ludwig Reiter
from datetime import date
import logging
import os.path
import tempfile
import re

from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.db import connection
from django.http import HttpResponse
from django.shortcuts import render
from django.utils.formats import date_format

import openpyxl

logger = logging.getLogger(__name__)


@login_required
def overview(request):
    return render(request, "fesreport.html")


@login_required
def download(request):
    return export_xlsx()


# Process template
def fill_sheet(ws):
    """Process the given worksheet.

    Replace all cell values fes:[indicator]:[gender]:[regioncategory]
    - indicator:      po04, a2_1, etc., also: a2_1+exit
                      Special Keys: entry, exit, target
    - gender:         m, f, t (male, female, total)
    - regioncategory: 1, 2+4, ...
    """

    FES = re.compile(
        r'fes:(?P<indicator>.+):(?P<gender>.+):(?P<regioncategory>.+)')
    for row in ws.rows:
        for cell in row:
            fes = FES.search(u"{}".format(cell.value))  # convert to string
            if fes:
                # indcator
                indicator = fes.group('indicator').lower()
                conditions = []
                for i in indicator.split('+'):
                    # rewrite the where condition
                    if i == 'exit':
                        conditions.append('completeness_exit = 100')
                    elif indicator == 'entry':
                        conditions.append('true')  # all entries are countable
                    else:
                        conditions.append(i)
                indicator_condition = ' AND '.join(conditions)

                # gender
                gender = fes.group('gender')[0].lower()
                # rewrite the where condition
                if gender == 'm':
                    gender = "= 1"  # male
                elif gender == 'f':
                    gender = "= 0"  # female
                else:
                    gender = 'in (0,1)'  # total

                # regioncategory
                rc = ','.join(fes.group('regioncategory').split('+'))

                # SELECT
                if indicator == 'target':
                    select = 'SELECT sum(oi_target_number) ' \
                             ' FROM beneficiary ' \
                             ' WHERE regioncategory_id in ({})'.format(rc)
                else:
                    select = 'SELECT count(*) ' \
                             ' FROM eval_virtual_participations ' \
                             ' WHERE {} AND gender {} ' \
                             ' AND regioncategory_id in ({})'.format(
                                 indicator_condition,
                                 gender,
                                 rc)

                # fetch result
                with connection.cursor() as cursor:
                    cursor.execute(select)
                    cell.value = cursor.fetchone()[0]


# work with the xlsx document.
def calculate_difference_in_months(start_date, end_date):
    """Calculates the difference between to dates
        end_date (date): Defaults to None. upper bound
        start_date (date): Defaults to None. [description]

    Every partial month is counted. 01.01.2017 - 01.01.2018
    is counted as 13 months.

    Returns:
        Integer: months
    """

    months_from_years = (end_date.year - start_date.year) * 12
    return months_from_years + end_date.month - start_date.month + 1


def open_workbook(filename):
    return openpyxl.load_workbook(filename=filename)


def get_sheet(workbook):
    # name of the sheet is template specific.
    return workbook[u"Sicht_Progr"]


def export_xlsx_as_string():
    with tempfile.NamedTemporaryFile() as tmpfile:
        workbook = open_workbook(get_template_xlsx())
        sheet = get_sheet(workbook)
        fill_sheet(sheet)
        write_metadate(sheet)
        workbook.save(tmpfile.name)
        tmpfile.seek(0)
        xlsx_str = tmpfile.read()
    return xlsx_str


def write_metadate(sheet):
    """Write the current date into the meta data section.

    All other dates and calculations are already in the template.
    """
    today = date.today()
    start = sheet['B4'].value.date()
    end = sheet['B5'].value.date()

    laufzeit_total = calculate_difference_in_months(start, end)
    laufzeit_aktuell = calculate_difference_in_months(start, today)
    anteil_laufzeit = float(laufzeit_aktuell) / float(laufzeit_total)
    sheet['B4'].value = date_format(start, "SHORT_DATE_FORMAT")
    sheet['B5'].value = date_format(end, "SHORT_DATE_FORMAT")
    sheet['B6'].value = date_format(today, "SHORT_DATE_FORMAT")
    sheet['D4'].value = laufzeit_total
    sheet['D5'].value = laufzeit_aktuell
    sheet['D6'].value = anteil_laufzeit


def export_xlsx():
    xlsx_str = export_xlsx_as_string()
    response = HttpResponse(xlsx_str,
                            content_type="application/vnd.openxmlformats-"
                                         "officedocument.spreadsheetml.sheet")
    result_name = "{}_Frueherkennung_{}.xlsx".format(
        settings.PROGRAM, str(date.today()))
    response['Content-Disposition'] = ('attachment; ' +
                                       'filename=' + result_name)
    return response


def get_template_xlsx():
    return os.path.join(settings.STATIC_ROOT, "documents",
                        settings.FRUEHERKENNUNG_TEMPLATE)
